<?php

/**
 * Builds and executes queries for the Redmine REST API.
 *
 * @todo Two: Support authentication by username & password too.
 *
 * @see http://www.redmine.org/projects/redmine/wiki/Rest_api
 * @see redmine_api_query()
 */
class redmine_query extends rest_api_query {
  /**
   * Constructor requires a valid API key and access point URL to initialize.
   *
   * These can be set with Drupal variables instead.  E.g. In settings.php:
   * @code
   * $conf['redmine_api_key'] = 'VALUE';
   * $conf['redmine_access_point'] = 'VALUE';
   * @code
   *
   * @param $api_key String
   *    The API key to authenticate with Redmine via the API.  Defaults to 
   *    variable_get('redmine_api_key')
   * @param $access_point String
   *    The Base URL to access the Redmine instance.  Defaults to 
   *    variable_get('redmine_access_point')
   */
  function __construct($api_key = NULL, $access_point = NULL) {
    $args = func_get_args();
    foreach (array('api_key', 'access_point') as $i => $key) {
      // Was the parameter passed in?
      if (isset($args[$i])) {
        // Yes;  Set the property.
        $this->$key = $args[$i];
      }
      else {
        // No;  Is it set in Drupal's configuration?
        $name = "redmine_$key";
        if ($variable = variable_get($name, FALSE)) {
          // Yes;  Set the property.
          $this->$key = $variable;
        }
        else {
          // No;  Fail.
          $args = array('!key' => $name, '!conf' => conf_path());
          $message = "Redmine API needs the <code>!key</code> to use the REST API.  Set <code>\$conf['!key'] = 'VALUE';</code> in <code>!conf/settings.php</code>.";

          // Set the message on this object, as a drupal message and with watchdog.
          $this->error = t($message, $args);
          drupal_set_message($this->error, 'error');
          watchdog('redmine', $message, $args, WATCHDOG_ERROR);

          // Initialization failed.  Stop trying.
          return;
        }
      }
    }

    // Initialization succeeded.  Permit this redmine_query object to be executed.
    $this->initialized = TRUE;
  }
}
